function [id] = RCL2_RUNTIME(O_Tt_DAT,D,D_labels,classnum,indTest,lambda,median_c,beta_c,mean_x,maxI,weight_thre)
% ========================================================================
% Regularized Robust Coding for Face Recogntion (RRC), Version 1.0
% Copyright(c) 2013  Meng YANG, Lei Zhang, Jian Yang and David Zhang
% All Rights Reserved.
%
% ----------------------------------------------------------------------
% Permission to use, copy, or modify this software and its documentation
% for educational and research purposes only and without fee is here
% granted, provided that this copyright notice and the original authors'
% names appear on all copies and supporting documentation. This program
% shall not be used, rewritten, or adapted as the basis of a commercial
% software or hardware product without first obtaining permission of the
% authors. The authors make no representations about the suitability of
% this software for any purpose. It is provided "as is" without express
% or implied warranty.
%----------------------------------------------------------------------
%
% This is an implementation of the algorithm for Regularized Robust Coding
% with l2-norm regularization of the coding vector
%
% Please refer to the following paper
%
% Meng Yang, Lei Zhang, Jian Yang, and David Zhang,"Regularized Robust Coding for Face Recogntion", 
% IEEE Trans. Image Processing, 2013.
% Meng Yang, Lei Zhang, Jian Yang, and David Zhang, "Robust sparse coding for face recognition", CVPR 2011.
%
% 
%----------------------------------------------------------------------
%
%Input : (1) O_Tt_DAT: the testing data matrix. 
%                      Each column is a testing sample
%        (2) D: the training data 
%        (3) D_labels: the labels of training data
%        (4) classnum: number of classes
%        (5) indTest:  index of testing sample
%        (6) lambda:  parameter to regularize coding vector
%        (7) median_c: parameter to control weight updating
%        (8) beta_c: parameter to control weight updating
%        (9) mean_x: vector of mean face of training samples
%        (10) maxI:  maximum iteration number to update the weight
%        (11) weight_thre: the threshold to automatically stop iteration
%
%
%Output: (1) id:  the identity of testing sample
%
%-----------------------------------------------------------------------
%
%Usage:
%Given a training data, testing data, and the
%parameters, opts, use the function like that in demo_AR
%
%
%-----------------------------------------------------------------------

num_D         = size(D,2);

y            = O_Tt_DAT(:,indTest);
y            = y./norm(y,2);

residual     =   (y-mean_x).^2;
residual_sort      =   sort(residual); 
iter               =   residual_sort(ceil(median_c*length(residual))); 

beta               =   beta_c/iter; 
w            =   1./(1+1./exp(-beta*(residual-iter)));

weight_pref = w;

for nit = 1: maxI
    fprintf('.');
    ww = w./max(w);
    index_w = find(ww>=1e-3);% remove the pixels with very low weight

    WW_index = repmat(w(index_w),[1 num_D]);
    W_D      = WW_index.*D(index_w,:);
    W_y      = w(index_w).*y(index_w);

    ratio = norm(W_y,2);
   
% when feature dimension is lager than number of dictionary atoms, the
% following coding will be faster

%     temp_s             =  (W_D'*W_D+lambda*ratio*eye(num_D))\(W_D'*W_y); % 
%     [temp_s, niters]   = cgsolve(W_D'*W_D+lambda*ratio*eye(num_D), W_D'*W_y); 

% when feature dimension is less than number of dictionary atoms, the
% following coding will be faster

      alpher             = (W_D*W_D'+lambda*ratio*eye(size(W_y,1)))\W_y; temp_s   = W_D'*alpher;
%       alpher             = cgsolve(W_D*W_D'+lambda*ratio*eye(size(W_y,1)),W_y); temp_s   = W_D'*alpher;
    
    residual           =   (y-D*temp_s).^2;
    residual_sort      =   sort(residual);     iter               =   residual_sort(ceil(median_c*length(residual))); 
    beta               =   beta_c/iter; 
    w                  =   1./(1+1./exp(-beta*(residual-iter)));

     weight_g  = norm(w-weight_pref,2)/norm(weight_pref,2);
     weight_pref = w;
%      weight_gap = [weight_gap weight_g];
     if weight_g < weight_thre
           nit
%          plot(weight_gap);keyboard;
         break;
     end
     
end

for class  =  1:classnum
      s           =   temp_s (D_labels == class);
      z1          =   w.*(y - D(:,D_labels == class)*s);
      gap1(class) = z1(:)'*z1(:);
end

index           =    find(gap1==min(gap1));
id              =    index(1);
