function [id] = Rl1C_RUNTIME1 (O_Tt_DAT,D,D_labels,classnum,indTest,lambda,median_c,beta_c,mean_x,maxI,weight_thre)
% ========================================================================
% Regularized Robust Coding for Face Recogntion (RRC), Version 1.0
% Copyright(c) 2013  Meng YANG, Lei Zhang, Jian Yang and David Zhang
% All Rights Reserved.
%
% ----------------------------------------------------------------------
% Permission to use, copy, or modify this software and its documentation
% for educational and research purposes only and without fee is here
% granted, provided that this copyright notice and the original authors'
% names appear on all copies and supporting documentation. This program
% shall not be used, rewritten, or adapted as the basis of a commercial
% software or hardware product without first obtaining permission of the
% authors. The authors make no representations about the suitability of
% this software for any purpose. It is provided "as is" without express
% or implied warranty.
%----------------------------------------------------------------------
%
% This is an implementation of the algorithm for Regularized Robust Coding
% with L1-norm regularization on the coding vector
%
% Please refer to the following paper
%
% Meng Yang, Lei Zhang, Jian Yang, and David Zhang,"Regularized Robust Coding for Face Recogntion", 
% IEEE Trans. Image Processing, 2013.
% Meng Yang, Lei Zhang, Jian Yang, and David Zhang, "Robust sparse coding for face recognition", CVPR 2011.
%
%----------------------------------------------------------------------
%
%Input : (1) O_Tt_DAT: the testing data matrix. 
%                      Each column is a testing sample
%        (2) D: the training data 
%        (3) D_labels: the labels of training data
%        (4) classnum: number of classes
%        (5) indTest:  index of testing sample
%        (6) lambda:  parameter to regularize coding vector
%        (7) median_c: parameter to control weight updating
%        (8) beta_c: parameter to control weight updating
%        (9) mean_x: vector of mean face of training samples
%        (10) maxI:  maximum iteration number to update the weight
%        (11) weight_thre: the threshold to automatically stop iteration
%
%
%Output: (1) id:  the identity of testing sample
%
%-----------------------------------------------------------------------
%
%Usage:
%Given a training data, testing data, and the
%parameters, opts, use the function like that in demo_AR
%
%
%-----------------------------------------------------------------------
dic_l         = size(D,2);

x_i0               =  zeros(dic_l,1);
w_i0               =  ones(dic_l,1);
x0                 =  ones(dic_l,1);

y            = O_Tt_DAT(:,indTest);
y            = y./norm(y,2);

residual     =   (y-mean_x).^2;
residual_sort      =   sort(residual); 
iter               =   residual_sort(ceil(median_c*length(residual))); 
beta               =   beta_c/iter; 
w            =   1./(1+1./exp(-beta*(residual-iter)));

weight_pref = w;


for nit = 1: maxI
    fprintf('.');
    ww = w./max(w);
    index_w = find(ww>=1e-3);

    WW_index = repmat(w(index_w),[1 dic_l]);
    W_D      = WW_index.*D(index_w,:);
    W_y = w(index_w).*y(index_w);
    f_l = size(W_y,1);

    ratio = norm(W_y,2);
    
    % inter loop to solving sparse coding vector with fixed pixel weight
    %--------------------begin of internal loop----------------------------%
      x_i               =  x_i0;
      w_i               =  w_i0;
      x                 =  x0;
      Kratio            =  0.01;
      innerit           =  0;
      yupu_pref         =  1000;
      
     
      newlambda         =  lambda*ratio;
      while norm(x-x_i,2)/norm(x,2) > 5e-2 & innerit <=20
             x_i          =  x;
             
             w_l = repmat(w_i,[1 f_l]); 
             [z, niters] =  cgsolve(W_D*(w_l.*W_D')+newlambda*eye(f_l), W_y); 
%              [z] =  (W_D*(w_l.*W_D')+newlambda*eye(f_l))\W_y; 
             x= w_l.*W_D'*z;
                  
             x_sort       =  sort(abs(x));
             yupu         =  abs(x_sort(ceil(Kratio*dic_l)));
             yupu         =  min(yupu/dic_l,yupu_pref);
             yupu_pref    =  yupu;
             w_i          =  (sqrt(x.^2+yupu.^2));
             innerit      =  innerit + 1;
%              plot(x);title([num2str(innerit) '-' num2str(norm(x-x_i,2)/norm(x,2))]);pause(1);
%              ER(innerit)  =    norm(x-x_i,2)/norm(x,2);
      end
      temp_s   =  x;
     
      %-------------------internal loop end----------------------------%
      
      residual           =   (y-D*temp_s).^2;
      residual_sort      =   sort(residual);     
      iter               =   residual_sort(ceil(median_c*length(residual))); 
      beta               =   beta_c/iter; 
      w                  =   1./(1+1./exp(-beta*(residual-iter)));

     weight_g  = norm(w-weight_pref,2)/norm(weight_pref,2);
     weight_pref = w;
%      weight_gap = [weight_gap weight_g];
     if weight_g < weight_thre
           nit
%          plot(weight_gap);keyboard;
         break;
     end
     
end

for class  =  1:classnum
      s           =   temp_s (D_labels == class);
      z1          =   w.*(y - D(:,D_labels == class)*s);
      gap1(class) = z1(:)'*z1(:);
end

index           =    find(gap1==min(gap1));
id              =    index(1);
