% ========================================================================
% Regularized Robust Coding for Face Recogntion (RRC), Version 1.0
% Copyright(c) 2013  Meng YANG, Lei Zhang, Jian Yang and David Zhang
% All Rights Reserved.
%
% ----------------------------------------------------------------------
% Permission to use, copy, or modify this software and its documentation
% for educational and research purposes only and without fee is here
% granted, provided that this copyright notice and the original authors'
% names appear on all copies and supporting documentation. This program
% shall not be used, rewritten, or adapted as the basis of a commercial
% software or hardware product without first obtaining permission of the
% authors. The authors make no representations about the suitability of
% this software for any purpose. It is provided "as is" without express
% or implied warranty.
%----------------------------------------------------------------------
%
% This is an demo of Regularized Robust Coding
% with l1-norm regularization of the coding vector for face recognition
% with disguise
%
% Please refer to the following paper
%
% Meng Yang, Lei Zhang, Jian Yang, and David Zhang,"Regularized Robust Coding for Face Recogntion", 
% IEEE Trans. Image Processing, 2013.
% Meng Yang, Lei Zhang, Jian Yang, and David Zhang, "Robust sparse coding for face recognition", CVPR 2011.
%
%----------------------------------------------------------------------

clear all;
clc;

%loading path
tem_fd = cd;
par.d_fd          =   [cd '/database/'];
addpath([cd '/RUNTIME/']);

%seting parameter
classnum        =   100;
par.nSample       =   7;
par.ID            =   [];
par.nameDatabase  =   'AR_disguise';

%loading data.
load([par.d_fd 'AR_database']);
Tr_DAT = []; trls = [];
for ci = 1:100
    Tr_DAT = [Tr_DAT Tr_dataMatrix(:,1+7*(ci-1)) Tr_dataMatrix(:,5+7*(ci-1):7+7*(ci-1))];
    trls   = [trls repmat(ci,[1 4])];
end
clear Tr_dataMatrix Tr_sampleLabels Tt_dataMatrix Tt_sampleLabels;

load([par.d_fd 'AR_database_Occlusion.mat']);
TtDATS1_sunglass = []; 
TtDATS2_sunglass = [];
ttls_sunglass = [];
TtDATS1_scarf = []; 
TtDATS2_scarf = [];
ttls_scarf = [];
for ci = 1:100
    TtDATS1_sunglass = [TtDATS1_sunglass Tr_dataMatrix(:,1+6*(ci-1):3+6*(ci-1))];
    TtDATS2_sunglass = [TtDATS2_sunglass Tt_dataMatrix(:,1+6*(ci-1):3+6*(ci-1))];
    ttls_sunglass   = [ttls_sunglass repmat(ci,[1 3])];
    
    TtDATS1_scarf = [TtDATS1_scarf Tr_dataMatrix(:,4+6*(ci-1):6+6*(ci-1))];
    TtDATS2_scarf = [TtDATS2_scarf Tt_dataMatrix(:,4+6*(ci-1):6+6*(ci-1))];
    ttls_scarf   = [ttls_scarf repmat(ci,[1 3])];
end
clear Tr_dataMatrix Tr_sampleLabels Tt_dataMatrix Tt_sampleLabels;

%testing data
Tt_DAT            =  TtDATS2_scarf;
ttls              =  ttls_scarf;

%dowmsample data
for i = 1:size(Tr_DAT,2)
    tem = reshape(Tr_DAT(:,i),[165 120]);
    tem1 = uint8(imresize(tem,[42 30]));
    O_Tr_DAT(:,i) = tem1(:);
end

Tem_DAT = [];
for i = 1:size(Tt_DAT,2)
    tem = reshape(Tt_DAT(:,i),[165 120]);
    tem1 = uint8(imresize(tem,[42 30]));
    O_Tt_DAT(:,i) = tem1(:);
end

D        = double(O_Tr_DAT);
Test_DAT = double(O_Tt_DAT);
D_labels = trls;
testlabels = ttls;

im_h       = 42;
im_w       = 30;

[disc_set,disc_value,Mean_Image]=Eigenface_f(D,260);
disc_value = sqrt((disc_value));
mean_x       =    (Mean_Image+0.001*disc_set*disc_value');

sigma      =  1e-1;

nit        =  1;
nIter      =  15;  % for simple, we just do 10 iterations.
BETA_A     =  [8];
MEDIAN_A   =  [0.6];

D          =  D./ repmat(sqrt(sum(D.*D)),[size(D,1) 1]);
beta_a     = BETA_A(1);
median_a   = MEDIAN_A(1);
eye_M      = eye(size(D,2));
dic_l      = size(D,2);
LAMBDA     = [0.0001];
weight_thre     = 5e-2;
weight_gap      = [];

for li = 1:size(LAMBDA,2)
    lambda = LAMBDA(li);
ID  =   [];
for pro_i = 1:size(Test_DAT,2)
    fprintf('%s\n',num2str(pro_i));
    y = double(Test_DAT(:,pro_i));

    %====================RRC_L1 begin=========================%
     residual           =   (y-mean_x).^2;
     residual_sort = sort(residual);
     iter = residual_sort(ceil(median_a*length(residual))); beta = beta_a/iter; 
     w = 1./(1+1./exp(-beta*(residual-iter)));
     weight_pref = w;

     norm_y_D = norm(y);
     y = y./norm(y);
     
     for nit = 1: nIter

     tem_w = w./max(w);
     index_w = find(tem_w>1e-3);
     % remove the pixels with very small weight
     W_y = w(index_w).*y(index_w);
     W_D = repmat(w(index_w),[1 size(D,2)]).*D(index_w,:);
     
    %---------------Internal sparse coding begin--------------%
      x_i               =  zeros(dic_l,1);
      w_i               =  ones(dic_l,1);
      x                 =  ones(dic_l,1);
      Kratio            =  0.01;
      innerit           =  0;
      yupu_pref         =  1000;
      WDWD              =  W_D'*W_D;
      WDWy              =  W_D'*W_y;
      newlambda         =  lambda*norm(W_y);
      while norm(x-x_i,2)/norm(x,2) > 1e-2 && innerit <=50
             x_i          =  x;
             w_l          =  repmat(w_i,[1 dic_l]);
             w_r          =  w_l';
             z            =  (WDWD.*w_r+newlambda*eye_M)\WDWy;
             x= w_i.*z; 
             x_sort       =  sort(abs(x));
             yupu         =  abs(x_sort(ceil(Kratio*dic_l)));
             yupu         =  min(yupu/dic_l,yupu_pref);
             yupu_pref    =  yupu;
             w_i          =  sqrt(x.^2+yupu.^2);
             innerit      =  innerit + 1;
     end
     temp_s   =  x;
      
     residual = norm_y_D^2.*(y-D*temp_s).^2;
          
     residual_sort = sort(residual);
     iter = residual_sort(ceil(median_a*length(residual))); beta = beta_a/iter; 
     w = 1./(1+1./exp(-beta*(residual-iter)));
     % w        =   sqrt(w);
%      imshow(reshape(w,[im_h, im_w]));title(num2str(nit));pause(1);

     weight_g  = norm(w-weight_pref,2)/norm(weight_pref,2);
     weight_pref = w;
     weight_gap = [weight_gap weight_g];
     if weight_g < weight_thre
           nit
%          plot(weight_gap);keyboard;
         break;
     end

     s_pref = temp_s;
     end

    for class = 1:classnum
    s = temp_s (D_labels == class);
    z1 = w.*(y - D(:,D_labels == class)*s);
    gap1(class) = z1(:)'*z1(:);
    end
    
    index = find(gap1==min(gap1));
    ID = [ID index(1)];
end

reco_rate  =  sum(ID==testlabels)/length(testlabels);
fid = fopen([cd '/RRCl1_ar_disguise2.txt'],'a');
fprintf(fid,'%s\n','-----------------session 2 :scarf---------------------------------------');
fprintf(fid,'%s%8f%s%8f%s%8f\n','nIter = ',nIter,...
    '   lambda = ',lambda,'   beta_a = ',beta_a);
fprintf(fid,'%s%8f%s%8f%s%8f\n\n','weight_thre = ', weight_thre,'median_a = ', median_a,...
    '   recognition rate', reco_rate);
fclose(fid);
end